@@ -18,6 +18,12 @@
      Boston, MA 02111-1307, USA.
 */
 
+/* This code is based in part on
+   unzip 1.00 Copyright 1998-2003 Gilles Vollant
+   http://www.winimage.com/zLibDll"
+*/
+
+
 #include "platform.h"
 #include <ctype.h>
 #include "extractor.h"
@@ -89,16 +95,11 @@ typedef struct unz_global_info_s
 #ifndef ALLOC
 # define ALLOC(size) (malloc(size))
 #endif
-#ifndef TRYFREE
-# define TRYFREE(p) {if (p) free(p);}
-#endif
 
 #define SIZECENTRALDIRITEM (0x2e)
 #define SIZEZIPLOCALHEADER (0x1e)
 
 
-const char unz_copyright[] =
-   " unzip 1.00 Copyright 1998-2003 Gilles Vollant - http://www.winimage.com/zLibDll";
 
 /* EXTRACTOR_unzip_file_info_interntal contain internal info about a file in zipfile*/
 typedef struct unz_file_info_internal_s
@@ -166,15 +167,10 @@ typedef struct
 */
 
 
-static int unzlocal_getByte OF((
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
-    voidpf filestream,
-    int *pi));
-
-static int unzlocal_getByte(pzlib_filefunc_def,filestream,pi)
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def;
-    voidpf filestream;
-    int *pi;
+static int 
+unzlocal_getByte(const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
+		 voidpf filestream,
+		 int *pi)
 {
     unsigned char c;
     int err = (int)ZREAD(*pzlib_filefunc_def,filestream,&c,1);
@@ -193,95 +189,80 @@ static int unzlocal_getByte(pzlib_filefunc_def,filestream,pi)
 }
 
 
-/* ===========================================================================
-   Reads a long in LSB order from the given gz_stream. Sets
-*/
-static int unzlocal_getShort OF((
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
-    voidpf filestream,
-    uLong *pX));
-
-static int unzlocal_getShort (pzlib_filefunc_def,filestream,pX)
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def;
-    voidpf filestream;
-    uLong *pX;
+static int 
+unzlocal_getShort (const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
+		   voidpf filestream,
+		   uLong *pX)
 {
-    uLong x ;
-    int i;
-    int err;
-
-    err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x = (uLong)i;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x += ((uLong)i)<<8;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        *pX = x;
-    else
-        *pX = 0;
+  uLong x;
+  int i;
+  int err;
+  
+  err = unzlocal_getByte(pzlib_filefunc_def, filestream, &i);
+  *pX = 0;
+  if (err != EXTRACTOR_UNZIP_OK)
     return err;
+  x = (uLong)i;
+  err = unzlocal_getByte(pzlib_filefunc_def, filestream, &i);
+  if (err != EXTRACTOR_UNZIP_OK)
+    return err;
+  x += ((uLong)i)<<8;  
+  *pX = x;
+  return err;
 }
 
-static int unzlocal_getLong OF((
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
-    voidpf filestream,
-    uLong *pX));
-
-static int unzlocal_getLong (pzlib_filefunc_def,filestream,pX)
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def;
-    voidpf filestream;
-    uLong *pX;
+static int 
+unzlocal_getLong (const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
+		  voidpf filestream,
+		  uLong *pX)
 {
-    uLong x ;
-    int i;
-    int err;
-
-    err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x = (uLong)i;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x += ((uLong)i)<<8;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x += ((uLong)i)<<16;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
-    x += ((uLong)i)<<24;
-
-    if (err==EXTRACTOR_UNZIP_OK)
-        *pX = x;
-    else
-        *pX = 0;
+  uLong x ;
+  int i;
+  int err;
+  
+  *pX = 0;
+  err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
+  if (err != EXTRACTOR_UNZIP_OK)
     return err;
+  x = (uLong)i;
+  err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
+  if (err != EXTRACTOR_UNZIP_OK)
+    return err;
+  x += ((uLong)i)<<8;
+  err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
+  if (err != EXTRACTOR_UNZIP_OK)
+    return err;
+  x += ((uLong)i)<<16;
+  err = unzlocal_getByte(pzlib_filefunc_def,filestream,&i);
+  if (err != EXTRACTOR_UNZIP_OK)
+    return err;
+  x += ((uLong)i)<<24;
+  *pX = x;
+  return err;
 }
 
 
 /* My own strcmpi / strcasecmp */
-static int strcmpcasenosensitive_internal (fileName1,fileName2)
-    const char* fileName1;
-    const char* fileName2;
+static int 
+strcmpcasenosensitive_internal (const char* fileName1,
+				const char* fileName2)
 {
-    for (;;)
+  while (1)
     {
-        char c1=*(fileName1++);
-        char c2=*(fileName2++);
-        if ((c1>='a') && (c1<='z'))
-            c1 -= 0x20;
-        if ((c2>='a') && (c2<='z'))
-            c2 -= 0x20;
-        if (c1=='\0')
-            return ((c2=='\0') ? 0 : -1);
-        if (c2=='\0')
-            return 1;
-        if (c1<c2)
-            return -1;
-        if (c1>c2)
-            return 1;
+      char c1=*(fileName1++);
+      char c2=*(fileName2++);
+      if ((c1>='a') && (c1<='z'))
+	c1 -= 0x20;
+      if ((c2>='a') && (c2<='z'))
+	c2 -= 0x20;
+      if (c1=='\0')
+	return ((c2=='\0') ? 0 : -1);
+      if (c2=='\0')
+	return 1;
+      if (c1<c2)
+	return -1;
+      if (c1>c2)
+	return 1;
     }
 }
 
@@ -305,35 +286,27 @@ static int strcmpcasenosensitive_internal (fileName1,fileName2)
         (like 1 on Unix, 2 on Windows)
 
 */
-int EXTRACTOR_common_unzip_string_file_name_compare (fileName1,fileName2,iCaseSensitivity)
-    const char* fileName1;
-    const char* fileName2;
-    int iCaseSensitivity;
+int 
+EXTRACTOR_common_unzip_string_file_name_compare (const char* fileName1,
+						 const char* fileName2,
+						 int iCaseSensitivity)
 {
-    if (iCaseSensitivity==0)
-        iCaseSensitivity=CASESENSITIVITYDEFAULTVALUE;
-
-    if (iCaseSensitivity==1)
-        return strcmp(fileName1,fileName2);
-
-    return STRCMPCASENOSENTIVEFUNCTION(fileName1,fileName2);
+  if (iCaseSensitivity==0)
+    iCaseSensitivity=CASESENSITIVITYDEFAULTVALUE;
+  
+  if (iCaseSensitivity==1)
+    return strcmp(fileName1,fileName2);
+  
+  return STRCMPCASENOSENTIVEFUNCTION(fileName1,fileName2);
 }
 
 #ifndef BUFREADCOMMENT
 #define BUFREADCOMMENT (0x400)
 #endif
 
-/*
-  Locate the Central directory of a zipfile (at the end, just before
-    the global comment)
-*/
-static uLong unzlocal_SearchCentralDir OF((
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
-    voidpf filestream));
-
-static uLong unzlocal_SearchCentralDir(pzlib_filefunc_def,filestream)
-    const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def;
-    voidpf filestream;
+static uLong 
+unzlocal_SearchCentralDir(const EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def,
+			  voidpf filestream)
 {
     unsigned char* buf;
     uLong uSizeFile;
@@ -384,16 +357,16 @@ static uLong unzlocal_SearchCentralDir(pzlib_filefunc_def,filestream)
         if (uPosFound!=0)
             break;
     }
-    TRYFREE(buf);
+    free(buf);
     return uPosFound;
 }
 
 /*
    Translate date/time from Dos format to EXTRACTOR_unzip_tm_unz (readable more easilty)
 */
-static void unzlocal_DosDateToTmuDate (ulDosDate, ptm)
-    uLong ulDosDate;
-    EXTRACTOR_unzip_tm_unz* ptm;
+static void 
+unzlocal_DosDateToTmuDate (uLong ulDosDate,
+			   EXTRACTOR_unzip_tm_unz* ptm)
 {
     uLong uDate;
     uDate = (uLong)(ulDosDate>>16);
@@ -408,21 +381,16 @@ static void unzlocal_DosDateToTmuDate (ulDosDate, ptm)
 
 
 
-static int unzlocal_GetCurrentFileInfoInternal (file,
-                                              pfile_info,
-                                              pfile_info_internal,
-                                              szFileName, fileNameBufferSize,
-                                              extraField, extraFieldBufferSize,
-                                              szComment,  commentBufferSize)
-    EXTRACTOR_unzip_file file;
-    EXTRACTOR_unzip_file_info *pfile_info;
-    unz_file_info_internal *pfile_info_internal;
-    char *szFileName;
-    uLong fileNameBufferSize;
-    void *extraField;
-    uLong extraFieldBufferSize;
-    char *szComment;
-    uLong commentBufferSize;
+static int 
+unzlocal_GetCurrentFileInfoInternal (EXTRACTOR_unzip_file file,
+				     EXTRACTOR_unzip_file_info *pfile_info,
+				     unz_file_info_internal *pfile_info_internal,
+				     char *szFileName,
+				     uLong fileNameBufferSize,
+				     void *extraField,
+				     uLong extraFieldBufferSize,
+				     char *szComment,
+				     uLong commentBufferSize)
 {
     unz_s* s;
     EXTRACTOR_unzip_file_info file_info;
@@ -574,8 +542,8 @@ static int unzlocal_GetCurrentFileInfoInternal (file,
   Set the current file of the zipfile to the first file.
   return UNZ_OK if there is no problem
 */
-int EXTRACTOR_common_unzip_go_to_first_file (file)
-    EXTRACTOR_unzip_file file;
+int 
+EXTRACTOR_common_unzip_go_to_first_file (EXTRACTOR_unzip_file file)
 {
     int err=EXTRACTOR_UNZIP_OK;
     unz_s* s;
@@ -601,9 +569,9 @@ int EXTRACTOR_common_unzip_go_to_first_file (file)
      Else, the return value is a EXTRACTOR_unzip_file Handle, usable with other function
        of this unzip package.
 */
-EXTRACTOR_unzip_file EXTRACTOR_common_unzip_open2 (path, pzlib_filefunc_def)
-    const char *path;
-    EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def;
+EXTRACTOR_unzip_file 
+EXTRACTOR_common_unzip_open2 (const char *path,
+			      EXTRACTOR_unzip_filefunc_def* pzlib_filefunc_def)
 {
     unz_s us;
     unz_s *s;
@@ -619,8 +587,6 @@ EXTRACTOR_unzip_file EXTRACTOR_common_unzip_open2 (path, pzlib_filefunc_def)
 
     int err=EXTRACTOR_UNZIP_OK;
 
-    if (unz_copyright[0]!=' ')
-        return NULL;
     memset (&us, 0, sizeof(us));	
     us.z_filefunc = *pzlib_filefunc_def;
 
@@ -704,8 +670,8 @@ EXTRACTOR_unzip_file EXTRACTOR_common_unzip_open2 (path, pzlib_filefunc_def)
   Close the file in zip opened with unzipOpenCurrentFile
   Return EXTRACTOR_UNZIP_CRCERROR if all the file was read but the CRC is not good
 */
-int EXTRACTOR_common_unzip_close_current_file (file)
-    EXTRACTOR_unzip_file file;
+int 
+EXTRACTOR_common_unzip_close_current_file (EXTRACTOR_unzip_file file)
 {
     int err=EXTRACTOR_UNZIP_OK;
 
@@ -728,13 +694,14 @@ int EXTRACTOR_common_unzip_close_current_file (file)
     }
 
 
-    TRYFREE(pfile_in_zip_read_info->read_buffer);
+    if (NULL != pfile_in_zip_read_info->read_buffer)
+      free(pfile_in_zip_read_info->read_buffer);
     pfile_in_zip_read_info->read_buffer = NULL;
     if (pfile_in_zip_read_info->stream_initialised)
         inflateEnd(&pfile_in_zip_read_info->stream);
 
     pfile_in_zip_read_info->stream_initialised = 0;
-    TRYFREE(pfile_in_zip_read_info);
+    free(pfile_in_zip_read_info);
 
     s->pfile_in_zip_read=NULL;
 
@@ -746,8 +713,7 @@ int EXTRACTOR_common_unzip_close_current_file (file)
   If there is files inside the .Zip opened with unzipOpenCurrentFile (see later),
     these files MUST be closed with unzipCloseCurrentFile before call unzipClose.
   return EXTRACTOR_UNZIP_OK if there is no problem. */
-int EXTRACTOR_common_unzip_close (file)
-    EXTRACTOR_unzip_file file;
+int EXTRACTOR_common_unzip_close (EXTRACTOR_unzip_file file)
 {
     unz_s* s;
     if (file==NULL)
@@ -758,7 +724,7 @@ int EXTRACTOR_common_unzip_close (file)
         EXTRACTOR_common_unzip_close_current_file(file);
 
     ZCLOSE(s->z_filefunc, s->filestream);
-    TRYFREE(s);
+    free(s);
     return EXTRACTOR_UNZIP_OK;
 }
 
@@ -768,19 +734,14 @@ int EXTRACTOR_common_unzip_close (file)
   No preparation of the structure is needed
   return EXTRACTOR_UNZIP_OK if there is no problem.
 */
-int EXTRACTOR_common_unzip_get_current_file_info (file,
-                                          pfile_info,
-                                          szFileName, fileNameBufferSize,
-                                          extraField, extraFieldBufferSize,
-                                          szComment,  commentBufferSize)
-    EXTRACTOR_unzip_file file;
-    EXTRACTOR_unzip_file_info *pfile_info;
-    char *szFileName;
-    uLong fileNameBufferSize;
-    void *extraField;
-    uLong extraFieldBufferSize;
-    char *szComment;
-    uLong commentBufferSize;
+int EXTRACTOR_common_unzip_get_current_file_info (EXTRACTOR_unzip_file file,
+						  EXTRACTOR_unzip_file_info *pfile_info,
+						  char *szFileName,
+						  uLong fileNameBufferSize,
+						  void *extraField,
+						  uLong extraFieldBufferSize,
+						  char *szComment,
+						  uLong commentBufferSize)
 {
     return unzlocal_GetCurrentFileInfoInternal(file,pfile_info,NULL,
                                                 szFileName,fileNameBufferSize,
@@ -793,8 +754,7 @@ int EXTRACTOR_common_unzip_get_current_file_info (file,
   return EXTRACTOR_UNZIP_OK if there is no problem
   return EXTRACTOR_UNZIP_END_OF_LIST_OF_FILE if the actual file was the latest.
 */
-int EXTRACTOR_common_unzip_go_to_next_file (file)
-    EXTRACTOR_unzip_file file;
+int EXTRACTOR_common_unzip_go_to_next_file (EXTRACTOR_unzip_file file)
 {
     unz_s* s;
     int err;
@@ -899,10 +859,10 @@ int EXTRACTOR_common_unzip_local_file (file, szFileName, iCaseSensitivity)
   return <0 with error code if there is an error
     (EXTRACTOR_UNZIP_ERRNO for IO error, or zLib error for uncompress error)
 */
-int EXTRACTOR_common_unzip_read_current_file (file, buf, len)
-    EXTRACTOR_unzip_file file;
-    voidp buf;
-    unsigned len;
+int 
+EXTRACTOR_common_unzip_read_current_file (EXTRACTOR_unzip_file file,
+					  voidp buf,
+					  unsigned len)
 {
     int err=EXTRACTOR_UNZIP_OK;
     uInt iRead = 0;
@@ -1040,18 +1000,16 @@ int EXTRACTOR_common_unzip_read_current_file (file, buf, len)
   store in *piSizeVar the size of extra info in local header
         (filename and size of extra field data)
 */
-static int unzlocal_CheckCurrentFileCoherencyHeader (s,piSizeVar,
-                                                    poffset_local_extrafield,
-                                                    psize_local_extrafield)
-    unz_s* s;
-    uInt* piSizeVar;
-    uLong *poffset_local_extrafield;
-    uInt  *psize_local_extrafield;
+static int 
+unzlocal_CheckCurrentFileCoherencyHeader (unz_s* s,
+					  uInt* piSizeVar,
+					  uLong *poffset_local_extrafield,
+					  uInt  *psize_local_extrafield)
 {
     uLong uMagic,uData,uFlags;
     uLong size_filename;
     uLong size_extra_field;
-    int err=EXTRACTOR_UNZIP_OK;
+    int err = EXTRACTOR_UNZIP_OK;
 
     *piSizeVar = 0;
     *poffset_local_extrafield = 0;
@@ -1059,21 +1017,13 @@ static int unzlocal_CheckCurrentFileCoherencyHeader (s,piSizeVar,
 
     if (ZSEEK(s->z_filefunc, s->filestream,s->cur_file_info_internal.offset_curfile +
                                 s->byte_before_the_zipfile,ZLIB_FILEFUNC_SEEK_SET)!=0)
-        return EXTRACTOR_UNZIP_ERRNO;
-
-
-    if (err==EXTRACTOR_UNZIP_OK) {
-        if (unzlocal_getLong(&s->z_filefunc, s->filestream,&uMagic) != EXTRACTOR_UNZIP_OK)
-            err=EXTRACTOR_UNZIP_ERRNO;
-        else if (uMagic!=0x04034b50)
-            err=EXTRACTOR_UNZIP_BADZIPFILE;
-    }
+      return EXTRACTOR_UNZIP_ERRNO;
+    if (unzlocal_getLong(&s->z_filefunc, s->filestream,&uMagic) != EXTRACTOR_UNZIP_OK)
+      err=EXTRACTOR_UNZIP_ERRNO;
+    else if (uMagic!=0x04034b50)
+      err=EXTRACTOR_UNZIP_BADZIPFILE;   
     if (unzlocal_getShort(&s->z_filefunc, s->filestream,&uData) != EXTRACTOR_UNZIP_OK)
         err=EXTRACTOR_UNZIP_ERRNO;
-/*
-    else if ((err==EXTRACTOR_UNZIP_OK) && (uData!=s->cur_file_info.wVersion))
-        err=EXTRACTOR_UNZIP_BADZIPFILE;
-*/
     if (unzlocal_getShort(&s->z_filefunc, s->filestream,&uFlags) != EXTRACTOR_UNZIP_OK)
         err=EXTRACTOR_UNZIP_ERRNO;
 
@@ -1132,11 +1082,11 @@ static int unzlocal_CheckCurrentFileCoherencyHeader (s,piSizeVar,
   Open for reading data the current file in the zipfile.
   If there is no error and the file is opened, the return value is EXTRACTOR_UNZIP_OK.
 */
-int EXTRACTOR_common_unzip_open_current_file3 (file, method, level, raw)
-    EXTRACTOR_unzip_file file;
-    int* method;
-    int* level;
-    int raw;
+int 
+EXTRACTOR_common_unzip_open_current_file3 (EXTRACTOR_unzip_file file,
+					   int* method,
+					   int* level,
+					   int raw)
 {
     int err=EXTRACTOR_UNZIP_OK;
     uInt iSizeVar;
@@ -1169,10 +1119,10 @@ int EXTRACTOR_common_unzip_open_current_file3 (file, method, level, raw)
     pfile_in_zip_read_info->raw=raw;
 
     if (pfile_in_zip_read_info->read_buffer==NULL)
-    {
-        TRYFREE(pfile_in_zip_read_info);
+      {
+	free(pfile_in_zip_read_info);
         return EXTRACTOR_UNZIP_INTERNALERROR;
-    }
+      }
 
     pfile_in_zip_read_info->stream_initialised=0;
 
