@@ -91,7 +91,7 @@ static void formatHelp(const char * general,
   int j;
   int ml;
   int p;
-  char * scp;
+  char scp[80];
   const char * trans;
 	
   printf(_("Usage: %s\n%s\n\n"),
@@ -128,8 +128,7 @@ static void formatHelp(const char * general,
   OUTER:
     while (ml - p > 78 - slen) {
       for (j=p+78-slen;j>p;j--) {
-	if (isspace(trans[j])) {
-	  scp = malloc(j-p+1);
+	if (isspace( (unsigned char) trans[j])) {
 	  memcpy(scp,
 		 &trans[p],
 		 j-p);
@@ -138,14 +137,12 @@ static void formatHelp(const char * general,
 		 scp,
 		 BORDER+2,
 		 "");
-	  free(scp);
 	  p = j+1;
 	  slen = BORDER+2;
 	  goto OUTER;
 	}
       }
       /* could not find space to break line */
-      scp = malloc(78 - slen + 1);
       memcpy(scp,
 	     &trans[p],
 	     78 - slen);
@@ -154,7 +151,6 @@ static void formatHelp(const char * general,
 	     scp,
 	     BORDER+2,
 	     "");	
-      free(scp);
       slen = BORDER+2;
       p = p + 78 - slen;
     }
@@ -230,6 +226,7 @@ print_selected_keywords (void *cls,
   char * keyword;
   iconv_t cd;
   const char *stype;
+  const char *mt;
 
   if (print[type] != YES)
     return 0;
@@ -237,7 +234,8 @@ print_selected_keywords (void *cls,
     fprintf (stdout,
 	     _("Found by `%s' plugin:\n"),
 	     plugin_name);
-  stype = gettext(EXTRACTOR_metatype_to_string(type));
+  mt = EXTRACTOR_metatype_to_string(type);
+  stype = (mt == NULL) ? _("unknown") : gettext(mt);
   switch (format)
     {
     case EXTRACTOR_METAFORMAT_UNKNOWN:
@@ -253,11 +251,14 @@ print_selected_keywords (void *cls,
 			       data);
       else
 	keyword = strdup(data);
-      fprintf (stdout,
-	       "%s - %s\n",
-	       stype,
-	       keyword);
-      free(keyword);
+      if (keyword != NULL)
+	{
+	  fprintf (stdout,
+		   "%s - %s\n",
+		   stype,
+		   keyword);
+	  free(keyword);
+	}
       if (cd != (iconv_t) -1)
 	iconv_close(cd);
       break;
@@ -309,9 +310,13 @@ print_selected_keywords_grep_friendly (void *cls,
 { 
   char * keyword;
   iconv_t cd;
+  const char *mt;
 
   if (print[type] != YES)
     return 0;
+  mt = EXTRACTOR_metatype_to_string(type);
+  if (mt == NULL)
+    mt = gettext_noop ("unknown");
   switch (format)
     {
     case EXTRACTOR_METAFORMAT_UNKNOWN:      
@@ -320,17 +325,20 @@ print_selected_keywords_grep_friendly (void *cls,
       if (verbose > 1)
 	fprintf (stdout,
 		 "%s: ",
-		 gettext(EXTRACTOR_metatype_to_string(type)));
+		 gettext(mt));
       cd = iconv_open(nl_langinfo(CODESET), "UTF-8");
       if (cd != (iconv_t) -1)
 	keyword = iconv_helper(cd,
 			       data);
       else
 	keyword = strdup(data);
-      fprintf (stdout,
-	       "'%s' ",
-	       keyword);
-      free(keyword);
+      if (keyword != NULL)
+	{
+	  fprintf (stdout,
+		   "`%s' ",
+		   keyword);
+	  free(keyword);
+	}
       if (cd != (iconv_t) -1)
 	iconv_close(cd);
       break;
@@ -340,9 +348,9 @@ print_selected_keywords_grep_friendly (void *cls,
       if (verbose > 1)
 	fprintf (stdout,
 		 "%s ",
-		 gettext(EXTRACTOR_metatype_to_string(type)));      
+		 gettext(mt));
       fprintf (stdout,
-	       "'%s'",
+	       "`%s'",
 	       data);
       break;
     default:
@@ -352,32 +360,6 @@ print_selected_keywords_grep_friendly (void *cls,
 }
 
 
-/**
- * Take title, auth, year and return a string
- */
-static char *
-str_splice(const char * title,
-	   const char * year,
-	   const char * auth) {
-  char * temp = malloc(16);
-  int i = 0;
-
-  snprintf(temp, 
-	   15,
-	   "%.5s%.5s%.5s", 
-	   auth, 
-	   year, 
-	   title);
-  for (i=0;i<strlen(temp);i++ ) {
-    if (! isalnum(temp[i]) )
-      temp[i] = '_';
-    else
-      temp[i] = tolower(temp[i]);
-  }
-  return temp;
-}
-
-
 /**
  * Entry in the map we construct for each file.
  */
@@ -497,8 +479,8 @@ static void
 finish_bibtex (const char *fn)
 {
   int i;
-  char *tya;
   const char *et;
+  char temp[20];
 
   if (entry_type != NULL)
     et = entry_type;
@@ -513,14 +495,22 @@ finish_bibtex (const char *fn)
 	     fn);
   else
     {
-      tya = str_splice (btm[0].value,
-			btm[1].value,
-			btm[2].value);      
+      snprintf (temp,
+		sizeof (temp),
+		"%.5s%.5s%.5s",
+		btm[2].value,
+		btm[1].value,
+		btm[0].value);
+      
+      for (i=strlen(temp)-1;i>=0;i-- )
+	if (! isalnum( (unsigned char) temp[i]) ) 
+	  temp[i] = '_';
+	else 
+	  temp[i] = tolower( (unsigned char) temp[i]);
       fprintf (stdout,
 	       "@%s %s { ",
 	       et,
-	       tya);
-      free (tya);
+	       temp);
     }
 
 	     
@@ -565,6 +555,13 @@ main (int argc, char *argv[])
   ignore_sigpipe ();
 #endif
   print = malloc (sizeof (int) * EXTRACTOR_metatype_get_max ());
+  if (print == NULL)
+    {
+      fprintf (stderr, 
+	       "malloc failed: %s\n",
+	       strerror (errno));
+      return 1;
+    }
   for (i = 0; i < EXTRACTOR_metatype_get_max (); i++)
     print[i] = YES;		/* default: print everything */
 
@@ -706,7 +703,13 @@ main (int argc, char *argv[])
 	  return -1;
 	}			/* end of parsing commandline */
     }				/* while (1) */
-
+  if (optind < 0)
+    {
+      fprintf (stderr,
+	       "Unknown error parsing options\n");
+      free (print);
+      return -1;
+    }
   if (argc - optind < 1)
     {
       fprintf (stderr,
