@@ -28,6 +28,16 @@
 #include <sys/shm.h>
 #endif
 #include <signal.h>
+#include <ltdl.h>
+
+#if HAVE_LIBBZ2
+#include <bzlib.h>
+#endif
+
+#if HAVE_ZLIB
+#include <zlib.h>
+#endif
+
 
 
 /**
@@ -54,21 +64,6 @@
 #define DEBUG 1
 
 
-#if HAVE_LTDL_H
-#include <ltdl.h>
-#else
-#include <../../libltdl/ltdl.h>
-#endif
-
-#if HAVE_LIBBZ2
-#include <bzlib.h>
-#endif
-
-#if HAVE_ZLIB
-#include <zlib.h>
-#endif
-
-
 /**
  * Linked list of extractor plugins.  An application builds this list
  * by telling libextractor to load various keyword-extraction
@@ -180,11 +175,13 @@ get_path_from_proc_exe() {
   char line[1024];
   char dir[1024];
   char * lnk;
-  size_t size;
+  char * ret;
+  char * lestr;
+  ssize_t size;
   FILE * f;
 
   snprintf(fn,
-	   64,
+	   sizeof (fn),
 	   "/proc/%u/maps",
 	   getpid());
   f = fopen(fn, "r");
@@ -193,9 +190,9 @@ get_path_from_proc_exe() {
       if ( (1 == sscanf(line,
 			"%*x-%*x %*c%*c%*c%*c %*x %*2x:%*2x %*u%*[ ]%s",
 			dir)) &&
-	   (NULL != strstr(dir,
-			   "libextractor")) ) {
-	strstr(dir, "libextractor")[0] = '\0';
+	   (NULL != (lestr = strstr(dir,
+				    "libextractor")) ) ) {
+	lestr[0] = '\0';
 	fclose(f);
 	return strdup(dir);
       }
@@ -203,12 +200,14 @@ get_path_from_proc_exe() {
     fclose(f);
   }
   snprintf(fn,
-	   64,
+	   sizeof (fn),
 	   "/proc/%u/exe",
 	   getpid());
   lnk = malloc(1029); /* 1024 + 5 for "lib/" catenation */
+  if (lnk == NULL)         
+    return NULL;
   size = readlink(fn, lnk, 1023);
-  if ( (size == 0) || (size >= 1024) ) {
+  if ( (size <= 0) || (size >= 1024) ) {
     free(lnk);
     return NULL;
   }
@@ -224,9 +223,14 @@ get_path_from_proc_exe() {
   }
   lnk[size] = '\0';
   lnk = cut_bin(lnk);
-  lnk = realloc(lnk, strlen(lnk) + 5);
-  strcat(lnk, "lib/"); /* guess "lib/" as the library dir */
-  return lnk;
+  ret = realloc(lnk, strlen(lnk) + 5);
+  if (ret == NULL)
+    {
+      free (lnk);
+      return NULL;
+    }
+  strcat(ret, "lib/"); /* guess "lib/" as the library dir */
+  return ret;
 }
 #endif
 
@@ -234,11 +238,15 @@ get_path_from_proc_exe() {
 /**
  * Try to determine path with win32-specific function
  */
-static char * get_path_from_module_filename() {
+static char * 
+get_path_from_module_filename() {
   char * path;
+  char * ret;
   char * idx;
 
   path = malloc(4103); /* 4096+nil+6 for "/lib/" catenation */
+  if (path == NULL)
+    return NULL;
   GetModuleFileName(NULL, path, 4096);
   idx = path + strlen(path);
   while ( (idx > path) &&
@@ -247,9 +255,14 @@ static char * get_path_from_module_filename() {
     idx--;
   *idx = '\0';
   path = cut_bin(path);
-  path = realloc(path, strlen(path) + 6);
-  strcat(path, "/lib/"); /* guess "lib/" as the library dir */
-  return path;
+  ret = realloc(path, strlen(path) + 6);
+  if (ret == NULL)
+    {
+      free (path);
+      return NULL;
+    }
+  strcat(ret, "/lib/"); /* guess "lib/" as the library dir */
+  return ret;
 }
 #endif
 
@@ -267,6 +280,8 @@ static char * get_path_from_dyld_image() {
       path = _dyld_get_image_name(i);
       if (path != NULL && strlen(path) > 0) {
         p = strdup(path);
+	if (p == NULL)
+	  return NULL;
         s = p + strlen(p);
         while ( (s > p) && (*s != '/') )
           s--;
@@ -291,6 +306,7 @@ get_path_from_PATH() {
   char * pos;
   char * end;
   char * buf;
+  char * ret;
   const char * p;
   size_t size;
 
@@ -298,7 +314,14 @@ get_path_from_PATH() {
   if (p == NULL)
     return NULL;
   path = strdup(p); /* because we write on it */
+  if (path == NULL)
+    return NULL;
   buf = malloc(strlen(path) + 20);
+  if (buf == NULL)
+    {
+      free (path);
+      return NULL;
+    }
   size = strlen(path);
   pos = path;
 
@@ -309,10 +332,17 @@ get_path_from_PATH() {
       pos = strdup(pos);
       free(buf);
       free(path);
+      if (pos == NULL)
+	return NULL;
       pos = cut_bin(pos);
-      pos = realloc(pos, strlen(pos) + 5);
-      strcat(pos, "lib/");
-      return pos;
+      ret = realloc(pos, strlen(pos) + 5);
+      if (ret == NULL)
+	{
+	  free (pos);
+	  return NULL;
+	}
+      strcat(ret, "lib/");
+      return ret;
     }
     pos = end + 1;
   }
@@ -321,10 +351,17 @@ get_path_from_PATH() {
     pos = strdup(pos);
     free(buf);
     free(path);
+    if (pos == NULL)
+      return NULL;
     pos = cut_bin(pos);
-    pos = realloc(pos, strlen(pos) + 5);
-    strcat(pos, "lib/");
-    return pos;
+    ret = realloc(pos, strlen(pos) + 5);
+    if (ret == NULL)
+      {
+	free (pos);
+	return NULL;
+      }
+    strcat(ret, "lib/");
+    return ret;
   }
   free(buf);
   free(path);
@@ -354,13 +391,18 @@ append_to_dir (const char *path,
 	       const char *fname)
 {
   char *ret;
+  size_t slen;
 
+  slen = strlen (path);
+  if (slen == 0)
+    return NULL;
   if (fname[0] == DIR_SEPARATOR)
     fname++;
-  ret = malloc (strlen (path) + strlen(fname) + 2);
-
+  ret = malloc (slen + strlen(fname) + 2);
+  if (ret == NULL)
+    return NULL;
 #ifdef MINGW
-  if (path[strlen(path)-1] == '\\')
+  if (path[slen-1] == '\\')
     sprintf (ret,
 	     "%s%s",
 	     path, 
@@ -371,7 +413,7 @@ append_to_dir (const char *path,
 	     path, 
 	     fname);
 #else
-  if (path[strlen(path)-1] == '/')
+  if (path[slen-1] == '/')
     sprintf (ret,
 	   "%s%s",
 	   path, 
@@ -407,6 +449,8 @@ get_installation_paths (PathProcessor pp,
   if (p != NULL)
     {
       d = strdup (p);
+      if (d == NULL)
+	return;
       prefix = strtok (d, ":");
       while (NULL != prefix)
 	{
@@ -434,10 +478,13 @@ get_installation_paths (PathProcessor pp,
   if (prefix == NULL)
     return;
   path = append_to_dir (prefix, PLUGINDIR);
-  if (0 != strcmp (path,
-		   PLUGININSTDIR))
-    pp (pp_cls, path);
-  free (path);
+  if (path != NULL)
+    {
+      if (0 != strcmp (path,
+		       PLUGININSTDIR))
+	pp (pp_cls, path);
+      free (path);
+    }
   free (prefix);
 }
 
@@ -484,6 +531,11 @@ find_plugin_in_path (void *cls,
 	continue;	
       sym_name++;
       sym = strdup (sym_name);
+      if (sym == NULL)
+	{
+	  closedir (dir);
+	  return;
+	}
       dot = strstr (sym, ".");
       if (dot != NULL)
 	*dot = '\0';
@@ -568,6 +620,11 @@ load_plugins_from_dir (void *cls,
 	continue;
       sym_name++;
       sym = strdup (sym_name);
+      if (NULL == sym)
+	{
+	  closedir (dir);
+	  return;
+	}
       dot = strstr (sym, ".");
       if (dot != NULL)
 	*dot = '\0';
@@ -641,10 +698,17 @@ get_symbol_with_prefix(void *lib_handle,
     return NULL;
   sym_name++;
   sym = strdup (sym_name);
+  if (sym == NULL)
+    return NULL;
   dot = strstr (sym, ".");
   if (dot != NULL)
     *dot = '\0';
   name = malloc(strlen(sym) + strlen(template) + 1);
+  if (name == NULL)
+    {
+      free (sym);
+      return NULL;
+    }
   sprintf(name,
 	  template,
 	  sym);
@@ -666,10 +730,11 @@ get_symbol_with_prefix(void *lib_handle,
 		  "`%s' and `%s'.\n",
 		  name+1,
 		  name,
-		  first_error,
+		  first_error == NULL ? "out of memory" : first_error,
 		  lt_dlerror());
 	}
-      free(first_error);
+      if (first_error != NULL)
+	free(first_error);
 #endif
     }
 
@@ -788,6 +853,11 @@ EXTRACTOR_plugin_add (struct EXTRACTOR_PluginList * prev,
   result = calloc (1, sizeof (struct EXTRACTOR_PluginList));
   result->next = prev;
   result->short_libname = strdup (library);
+  if (result->short_libname == NULL)
+    {
+      free (result);
+      return NULL;
+    }
   result->libname = libname;
   result->flags = flags;
   if (NULL != options)
@@ -828,6 +898,8 @@ EXTRACTOR_plugin_add_config (struct EXTRACTOR_PluginList * prev,
     return prev;
   len = strlen(config);
   cpy = strdup(config);
+  if (cpy == NULL)
+    return prev;
   pos = 0;
   last = 0;
   lastconf = 0;
@@ -1109,6 +1181,12 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
   HANDLE map;
 #endif
 
+  if (plugin == NULL)
+    {
+      close (in);
+      close (out);
+      return;
+    }
   if (0 != plugin_load (plugin))
     {
       close (in);
@@ -1142,6 +1220,12 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
 
   memset (&hdr, 0, sizeof (hdr));
   fin = fdopen (in, "r");
+  if (fin == NULL)
+    {
+      close (in);
+      close (out);
+      return;
+    }
   while (NULL != fgets (hfn, sizeof(hfn), fin))
     {
       if (strlen (hfn) <= 1)
@@ -1165,7 +1249,7 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
 
 #ifndef WINDOWS
       if ( (-1 != (shmid = shm_open (fn, O_RDONLY, 0))) &&
-	   (((size_t)-1) != (size = lseek (shmid, 0, SEEK_END))) &&
+	   (SIZE_MAX != (size = lseek (shmid, 0, SEEK_END))) &&
 	   (NULL != (ptr = mmap (NULL, size, PROT_READ, MAP_SHARED, shmid, 0))) &&
 	   (ptr != (void*) -1) )
 #else
@@ -1221,11 +1305,9 @@ write_plugin_data (int fd, const struct EXTRACTOR_PluginList *plugin)
   i = strlen (plugin->libname) + 1;
   write (fd, &i, sizeof (size_t));
   write (fd, plugin->libname, i);
-
   i = strlen (plugin->short_libname) + 1;
   write (fd, &i, sizeof (size_t));
   write (fd, plugin->short_libname, i);
-
   if (plugin->plugin_options != NULL)
     {
       i = strlen (plugin->plugin_options) + 1;
@@ -1233,11 +1315,11 @@ write_plugin_data (int fd, const struct EXTRACTOR_PluginList *plugin)
     }
   else
     {
-      i = 1;
-      str = "";
+      i = 0;
     }
   write (fd, &i, sizeof (size_t));
-  write (fd, str, i);
+  if (i > 0)
+    write (fd, str, i);
 }
 
 static struct EXTRACTOR_PluginList *
@@ -1247,34 +1329,62 @@ read_plugin_data (int fd)
   size_t i;
 
   ret = malloc (sizeof (struct EXTRACTOR_PluginList));
-
+  if (ret == NULL)
+    return NULL;
   read (fd, &i, sizeof (size_t));
   ret->libname = malloc (i);
+  if (ret->libname == NULL)
+    {
+      free (ret);
+      return NULL;
+    }
   read (fd, ret->libname, i);
 
   read (fd, &i, sizeof (size_t));
   ret->short_libname = malloc (i);
+  if (ret->short_libname == NULL)
+    {
+      free (ret->libname);
+      free (ret);
+      return NULL;
+    }
   read (fd, ret->short_libname, i);
 
   read (fd, &i, sizeof (size_t));
-  ret->plugin_options = malloc (i);
-  read (fd, ret->plugin_options, i);
-
+  if (i == 0)
+    {
+      ret->plugin_options = NULL;
+    }
+  else
+    {
+      ret->plugin_options = malloc (i);
+      if (ret->plugin_options == NULL)
+	{
+	  free (ret->short_libname);
+	  free (ret->libname);
+	  free (ret);
+	  return NULL;
+	}
+      read (fd, ret->plugin_options, i);
+    }
   return ret;
 }
 
 
 void CALLBACK 
-RundllEntryPoint(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow)
+RundllEntryPoint(HWND hwnd, 
+		 HINSTANCE hinst, 
+		 LPSTR lpszCmdLine, 
+		 int nCmdShow)
 {
-  int in, out;
+  int in;
+  int out;
 
   sscanf(lpszCmdLine, "%u %u", &in, &out);
-
   setmode (in, _O_BINARY);
   setmode (out, _O_BINARY);
-
-  process_requests (read_plugin_data (in), in, out);
+  process_requests (read_plugin_data (in),
+		    in, out);
 }
 #endif
 
@@ -1288,14 +1398,15 @@ start_process (struct EXTRACTOR_PluginList *plugin)
   int p1[2];
   int p2[2];
   pid_t pid;
+  int status;
 
+  plugin->cpid = -1;
 #ifndef WINDOWS
   if (0 != pipe (p1))
 #else
     if (0 != _pipe (p1, 0, _O_BINARY))
 #endif
     {
-      plugin->cpid = -1;
       plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return;
     }
@@ -1307,7 +1418,6 @@ start_process (struct EXTRACTOR_PluginList *plugin)
     {
       close (p1[0]);
       close (p1[1]);
-      plugin->cpid = -1;
       plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return;
     }
@@ -1334,14 +1444,13 @@ start_process (struct EXTRACTOR_PluginList *plugin)
       close (p1[1]);
       close (p2[0]);
       close (p2[1]);
-      plugin->cpid = -1;
       plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return;
     }
   if (pid == 0)
     {
-    close (p1[1]);
-    close (p2[0]);
+      close (p1[1]);
+      close (p2[0]);
       process_requests (plugin, p1[0], p2[1]);
       _exit (0);
     }
@@ -1350,7 +1459,16 @@ start_process (struct EXTRACTOR_PluginList *plugin)
   close (p2[1]);
   plugin->cpipe_in = fdopen (p1[1], "w");
   if (plugin->cpipe_in == NULL)
-    perror ("fdopen");
+    {
+      perror ("fdopen");
+      (void) kill (plugin->cpid, SIGKILL);
+      waitpid (plugin->cpid, &status, 0);
+      close (p1[1]);
+      close (p2[0]);
+      plugin->cpid = -1;
+      plugin->flags = EXTRACTOR_OPTION_DISABLED;
+      return;
+    }
   plugin->cpipe_out = p2[0];
 }
 
@@ -1532,8 +1650,8 @@ make_shm (int is_tail,
       CloseHandle (*mappedFile);
       return 1;
     }
-#endif
   return 0;
+#endif
 }
 
 
@@ -1757,6 +1875,7 @@ decompress_and_extract (struct EXTRACTOR_PluginList *plugins,
 			EXTRACTOR_MetaDataProcessor proc,
 			void *proc_cls) {
   unsigned char * buf;
+  unsigned char * rbuf;
   size_t dsize;
 #if HAVE_ZLIB
   z_stream strm;
@@ -1888,7 +2007,14 @@ decompress_and_extract (struct EXTRACTOR_PluginList *plugins,
 		    dsize *= 2;
 		    if (dsize > MAX_DECOMPRESS)
 		      dsize = MAX_DECOMPRESS;
-		    buf = realloc(buf, dsize);
+		    rbuf = realloc(buf, dsize);
+		    if (rbuf == NULL)
+		      {
+			free (buf);
+			buf = NULL;
+			break;
+		      }
+		    buf = rbuf;
 		    strm.next_out = (Bytef*) &buf[pos];
 		    strm.avail_out = dsize - pos;
 		  }
@@ -1902,10 +2028,12 @@ decompress_and_extract (struct EXTRACTOR_PluginList *plugins,
 			(ret != Z_STREAM_END) );
 	    dsize = pos + strm.total_out;
 	    inflateEnd(&strm);
-	    if (dsize == 0) {
-	      free(buf);
-	      buf = NULL;
-	    }
+	    if ( (dsize == 0) &&
+		 (buf != NULL) )
+	      {
+		free(buf);
+		buf = NULL;
+	      }
 	  }
       }
     }
@@ -1957,7 +2085,14 @@ decompress_and_extract (struct EXTRACTOR_PluginList *plugins,
 		    dsize *= 2;
 		    if (dsize > MAX_DECOMPRESS)
 		      dsize = MAX_DECOMPRESS;
-		    buf = realloc(buf, dsize);
+		    rbuf = realloc(buf, dsize);
+		    if (rbuf == NULL)
+		      {
+			free (buf);
+			buf = NULL;
+			break;
+		      }
+		    buf = rbuf;
 		    bstrm.next_out = (char*) &buf[bpos];
 		    bstrm.avail_out = dsize - bpos;
 		  } 
@@ -1971,7 +2106,8 @@ decompress_and_extract (struct EXTRACTOR_PluginList *plugins,
 			(bret != BZ_STREAM_END) );
 	      dsize = bpos + bstrm.total_out_lo32;
 	      BZ2_bzDecompressEnd(&bstrm);
-	      if (dsize == 0) 
+	      if ( (dsize == 0) &&
+		   (buf != NULL) )
 		{
 		  free(buf);
 		  buf = NULL;
