@@ -28,6 +28,8 @@ stndup (const char *str, size_t n)
 {
   char *tmp;
   tmp = malloc (n + 1);
+  if (tmp == NULL)
+    return NULL;
   tmp[n] = '\0';
   memcpy (tmp, str, n);
   return tmp;
@@ -79,7 +81,7 @@ static struct
 
 
 #define ADD(t,s) do { if (0 != (ret = proc (proc_cls, "tar", t, EXTRACTOR_METAFORMAT_UTF8, "text/plain", s, strlen(s)+1))) goto FINISH; } while (0)
-#define ADDF(t,s) do { if (0 != (ret = proc (proc_cls, "tar", t, EXTRACTOR_METAFORMAT_UTF8, "text/plain", s, strlen(s)+1))) { free(s); goto FINISH; } free (s); } while (0)
+#define ADDF(t,s) do { if ( (s != NULL) && (0 != (ret = proc (proc_cls, "tar", t, EXTRACTOR_METAFORMAT_UTF8, "text/plain", s, strlen(s)+1))) ) { free(s); goto FINISH; } if (s != NULL) free (s); } while (0)
 
 
 static int
@@ -98,6 +100,8 @@ processtEXt (const char *data,
   if (off >= length)
     return 0;                /* failed to find '\0' */
   keyword = EXTRACTOR_common_convert_to_utf8 (&data[off], length - off, "ISO-8859-1");
+  if (keyword == NULL)
+    return 0;
   i = 0;
   ret = 0;
   while (tagmap[i].name != NULL)
@@ -128,6 +132,7 @@ processiTXt (const char *data,
   int i;
   int compressed;
   char *buf;
+  char *lan;
   uLongf bufLen;
   int ret;
   int zret;
@@ -141,15 +146,21 @@ processiTXt (const char *data,
   language = &data[pos];
   ret = 0;
   if (stnlen (language, length - pos) > 0)
-    ADDF (EXTRACTOR_METATYPE_LANGUAGE,
-	  stndup (language, length - pos));
+    {
+      lan = stndup (language, length - pos);
+      ADDF (EXTRACTOR_METATYPE_LANGUAGE,
+	    lan);
+    }
   pos += stnlen (language, length - pos) + 1;
   if (pos + 1 >= length)
     return 0;
   translated = &data[pos];      /* already in utf-8! */
   if (stnlen (translated, length - pos) > 0)
-    ADDF (EXTRACTOR_METATYPE_KEYWORDS,
-	  stndup (translated, length - pos));
+    {
+      lan = stndup (translated, length - pos);
+      ADDF (EXTRACTOR_METATYPE_KEYWORDS,
+	    lan);
+    }
   pos += stnlen (translated, length - pos) + 1;
   if (pos >= length)
     return 0;
@@ -240,8 +251,8 @@ processzTXt (const char *data,
   int i;
   char *buf;
   uLongf bufLen;
-  int ret;
   int zret;
+  int ret;
 
   data += 4;
   off = stnlen (data, length) + 1;
@@ -250,7 +261,7 @@ processzTXt (const char *data,
   if (data[off] != 0)
     return 0;                /* compression method must be 0 */
   off++;
-
+  ret = 0;
   bufLen = 1024 + 2 * (length - off);
   while (1)
     {
