@@ -88,7 +88,7 @@ EXTRACTOR_zip_extract (const unsigned char *data,
   void *tmp;
   zip_entry * info;
   zip_entry * start;
-  char *filecomment = NULL;
+  char *filecomment;
   const unsigned char *pos;
   unsigned int offset, stop;
   unsigned int name_length, extra_length, comment_length;
@@ -152,14 +152,13 @@ EXTRACTOR_zip_extract (const unsigned char *data,
     pos = &data[offset--];
   if (offset == stop)
     {
-      
 #if DEBUG_EXTRACT_ZIP
-        fprintf (stderr,
+      fprintf (stderr,
                  "Did not find end of central directory structure signature. offset: %i\n",
                  offset);
       
-#endif  /*  */
-        return 0;
+#endif
+      return 0;
     }
   
     /* offset should now point to the start of the end-of-central directory structure */ 
@@ -174,29 +173,26 @@ EXTRACTOR_zip_extract (const unsigned char *data,
   if (filecomment_length > 0)
     {
       filecomment = malloc (filecomment_length + 1);
-      memcpy (filecomment, &pos[22], filecomment_length);
-      filecomment[filecomment_length] = '\0';
+      if (filecomment != NULL)
+	{
+	  memcpy (filecomment, &pos[22], filecomment_length);
+	  filecomment[filecomment_length] = '\0';
+	}
     }
+
+#if DEBUG_EXTRACT_ZIP
   if ((0 != pos[4]) && (0 != pos[5]))
-    {
+    fprintf (stderr,
+	     "WARNING: This seems to be the last disk in a multi-volume"
+	     " ZIP archive, and so this might not work.\n");          
+#endif
       
 #if DEBUG_EXTRACT_ZIP
-        fprintf (stderr,
-                 "WARNING: This seems to be the last disk in a multi-volume"
-                  " ZIP archive, and so this might not work.\n");
-      
-#endif  /*  */
-    }
   if ((pos[8] != pos[10]) && (pos[9] != pos[11]))
-    {
-      
-#if DEBUG_EXTRACT_ZIP
-        fprintf (stderr,
-                 "WARNING: May not be able to find all the files in this" 
-                 " ZIP archive (no multi-volume support right now).\n");
-      
-#endif  /*  */
-    }
+    fprintf (stderr,
+	     "WARNING: May not be able to find all the files in this" 
+	     " ZIP archive (no multi-volume support right now).\n");  
+#endif
   entry_total = pos[10] + (pos[11] << 8);
   entry_count = 0;
   
@@ -243,18 +239,17 @@ EXTRACTOR_zip_extract (const unsigned char *data,
      */ 
     if (!(('P' == pos[0]) && ('K' == pos[1]) && (0x01 == pos[2])
           && (0x02 == pos[3])))
-    {
-      
+      {      
 #if DEBUG_EXTRACT_ZIP
         fprintf (stderr,
                  "Did not find central directory structure signature. offset: %i\n",
                  offset);
-      
+	
 #endif
         if (filecomment != NULL)
 	  free (filecomment);
 	return 0;
-    }
+      }
   start = NULL;
   info = NULL;
   
@@ -275,18 +270,21 @@ EXTRACTOR_zip_extract (const unsigned char *data,
         fprintf (stderr, "Found filename length %i  Comment length: %i\n",
                  name_length, comment_length);
       
-#endif  /*  */
-        
+#endif        
         /* yay, finally get filenames */ 
         if (start == NULL)
         {
           start = malloc (sizeof (zip_entry));
+	  if (start == NULL)
+	    break;
           start->next = NULL;
           info = start;
         }
       else
         {
           info->next = malloc (sizeof (zip_entry));
+	  if (info->next == NULL)
+	    break;
           info = info->next;
           info->next = NULL;
         }
@@ -294,35 +292,35 @@ EXTRACTOR_zip_extract (const unsigned char *data,
       info->comment = malloc (comment_length + 1);
       
         /* (strings in zip files are not null terminated) */ 
-        memcpy (info->filename, &pos[46], name_length);
-      info->filename[name_length] = '\0';
-      memcpy (info->comment, &pos[46 + name_length + extra_length],
-               comment_length);
-      info->comment[comment_length] = '\0';
-      
+      if (info->filename != NULL)
+	{
+	  memcpy (info->filename, &pos[46], name_length);
+	  info->filename[name_length] = '\0';
+	}
+      if (info->comment != NULL)
+	{
+	  memcpy (info->comment, &pos[46 + name_length + extra_length],
+		  comment_length);
+	  info->comment[comment_length] = '\0';
+	}
+      offset += 46 + name_length + extra_length + comment_length;
+      pos = &data[offset];      
+      /* check for next header entry (0x02014b50) or (0x06054b50) if at end */ 
+      if (('P' != pos[0]) && ('K' != pos[1]))
+        {         
 #if DEBUG_EXTRACT_ZIP
-        fprintf (stderr, "Found file %s, Comment: %s\n", info->filename,
-                 info->comment);
-      
-#endif
-        offset += 46 + name_length + extra_length + comment_length;
-      pos = &data[offset];
-      
-        /* check for next header entry (0x02014b50) or (0x06054b50) if at end */ 
-        if (('P' != pos[0]) && ('K' != pos[1]))
-        {
-          
-#if DEBUG_EXTRACT_ZIP
-            fprintf (stderr,
-                     "Did not find next header in central directory.\n");
+	  fprintf (stderr,
+		   "Did not find next header in central directory.\n");
           
 #endif
-            info = start;
+	  info = start;
           while (info != NULL)
             {
               start = info->next;
-              free (info->filename);
-              free (info->comment);
+	      if (info->filename != NULL)
+		free (info->filename);
+	      if (info->comment != NULL)
+		free (info->comment);
               free (info);
               info = start;
             }
@@ -337,15 +335,11 @@ EXTRACTOR_zip_extract (const unsigned char *data,
     
     /* TODO: should this return an error? indicates corrupt zipfile (or
        disk missing in middle of multi-disk)? */ 
-    if (entry_count != entry_total)
-    {
-      
 #if DEBUG_EXTRACT_ZIP
-        fprintf (stderr,
-                 "WARNING: Did not find all of the zipfile entries that we should have.\n");
-      
-#endif  /*  */
-    }
+  if (entry_count != entry_total)
+    fprintf (stderr,
+	     "WARNING: Did not find all of the zipfile entries that we should have.\n");    
+#endif
   
   ret = proc (proc_cls,
 	      "zip",
@@ -364,7 +358,8 @@ EXTRACTOR_zip_extract (const unsigned char *data,
 		  filecomment,
 		  strlen (filecomment)+1);
     }
-  free (filecomment);
+  if (filecomment != NULL)
+    free (filecomment);
 
   
   /* if we've gotten to here then there is at least one zip entry (see get_zipinfo call above) */ 
@@ -384,6 +379,7 @@ EXTRACTOR_zip_extract (const unsigned char *data,
 			  info->filename,
 			  strlen (info->filename)+1);
             }
+	   free (info->filename);
         }
       if (info->comment != NULL)
 	{
@@ -397,9 +393,8 @@ EXTRACTOR_zip_extract (const unsigned char *data,
 			  info->comment,
 			  strlen (info->comment)+1);
 	    }
+	  free (info->comment);
 	}
-      free (info->filename);
-      free (info->comment);
       tmp = info;
       info = info->next;
       free (tmp);
